# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    PhoneNumberDetailed,
    PhoneNumberDeleteResponse,
    PhoneNumberUpdateResponse,
    PhoneNumberRetrieveResponse,
    PhoneNumberSlimListResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPhoneNumbers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        phone_number = client.phone_numbers.retrieve(
            "1293384261075731499",
        )
        assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.phone_numbers.with_raw_response.retrieve(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.phone_numbers.with_streaming_response.retrieve(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.phone_numbers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        phone_number = client.phone_numbers.update(
            phone_number_id="1293384261075731499",
        )
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        phone_number = client.phone_numbers.update(
            phone_number_id="1293384261075731499",
            address_id="dc8f39ac-953d-4520-b93b-786ae87db0da",
            billing_group_id="dc8e4d67-33a0-4cbb-af74-7b58f05bd494",
            connection_id="dc8e4d67-33a0-4cbb-af74-7b58f05bd494",
            customer_reference="customer-reference",
            external_pin="1234",
            hd_voice_enabled=True,
            tags=["tag"],
        )
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.phone_numbers.with_raw_response.update(
            phone_number_id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.phone_numbers.with_streaming_response.update(
            phone_number_id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number_id` but received ''"):
            client.phone_numbers.with_raw_response.update(
                phone_number_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        phone_number = client.phone_numbers.list()
        assert_matches_type(SyncDefaultFlatPagination[PhoneNumberDetailed], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        phone_number = client.phone_numbers.list(
            filter={
                "billing_group_id": "62e4bf2e-c278-4282-b524-488d9c9c43b2",
                "connection_id": "1521916448077776306",
                "country_iso_alpha2": "US",
                "customer_reference": "customer_reference",
                "emergency_address_id": "9102160989215728032",
                "number_type": {"eq": "local"},
                "phone_number": "phone_number",
                "source": "ported",
                "status": "active",
                "tag": "tag",
                "voice_connection_name": {
                    "contains": "test",
                    "ends_with": "test",
                    "eq": "test",
                    "starts_with": "test",
                },
                "voice_usage_payment_method": "channel",
                "without_tags": "true",
            },
            page_number=0,
            page_size=0,
            sort="connection_name",
        )
        assert_matches_type(SyncDefaultFlatPagination[PhoneNumberDetailed], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.phone_numbers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[PhoneNumberDetailed], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.phone_numbers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[PhoneNumberDetailed], phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        phone_number = client.phone_numbers.delete(
            "1293384261075731499",
        )
        assert_matches_type(PhoneNumberDeleteResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.phone_numbers.with_raw_response.delete(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(PhoneNumberDeleteResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.phone_numbers.with_streaming_response.delete(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(PhoneNumberDeleteResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.phone_numbers.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_slim_list(self, client: Telnyx) -> None:
        phone_number = client.phone_numbers.slim_list()
        assert_matches_type(SyncDefaultFlatPagination[PhoneNumberSlimListResponse], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_slim_list_with_all_params(self, client: Telnyx) -> None:
        phone_number = client.phone_numbers.slim_list(
            filter={
                "billing_group_id": "62e4bf2e-c278-4282-b524-488d9c9c43b2",
                "connection_id": "1521916448077776306",
                "country_iso_alpha2": "US",
                "customer_reference": "customer_reference",
                "emergency_address_id": "9102160989215728032",
                "number_type": {"eq": "local"},
                "phone_number": "phone_number",
                "source": "ported",
                "status": "active",
                "tag": "tag",
                "voice_connection_name": {
                    "contains": "test",
                    "ends_with": "test",
                    "eq": "test",
                    "starts_with": "test",
                },
                "voice_usage_payment_method": "channel",
            },
            include_connection=True,
            include_tags=True,
            page_number=0,
            page_size=0,
            sort="connection_name",
        )
        assert_matches_type(SyncDefaultFlatPagination[PhoneNumberSlimListResponse], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_slim_list(self, client: Telnyx) -> None:
        response = client.phone_numbers.with_raw_response.slim_list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[PhoneNumberSlimListResponse], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_slim_list(self, client: Telnyx) -> None:
        with client.phone_numbers.with_streaming_response.slim_list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[PhoneNumberSlimListResponse], phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncPhoneNumbers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.phone_numbers.retrieve(
            "1293384261075731499",
        )
        assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_numbers.with_raw_response.retrieve(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_numbers.with_streaming_response.retrieve(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.phone_numbers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.phone_numbers.update(
            phone_number_id="1293384261075731499",
        )
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.phone_numbers.update(
            phone_number_id="1293384261075731499",
            address_id="dc8f39ac-953d-4520-b93b-786ae87db0da",
            billing_group_id="dc8e4d67-33a0-4cbb-af74-7b58f05bd494",
            connection_id="dc8e4d67-33a0-4cbb-af74-7b58f05bd494",
            customer_reference="customer-reference",
            external_pin="1234",
            hd_voice_enabled=True,
            tags=["tag"],
        )
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_numbers.with_raw_response.update(
            phone_number_id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_numbers.with_streaming_response.update(
            phone_number_id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number_id` but received ''"):
            await async_client.phone_numbers.with_raw_response.update(
                phone_number_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.phone_numbers.list()
        assert_matches_type(AsyncDefaultFlatPagination[PhoneNumberDetailed], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.phone_numbers.list(
            filter={
                "billing_group_id": "62e4bf2e-c278-4282-b524-488d9c9c43b2",
                "connection_id": "1521916448077776306",
                "country_iso_alpha2": "US",
                "customer_reference": "customer_reference",
                "emergency_address_id": "9102160989215728032",
                "number_type": {"eq": "local"},
                "phone_number": "phone_number",
                "source": "ported",
                "status": "active",
                "tag": "tag",
                "voice_connection_name": {
                    "contains": "test",
                    "ends_with": "test",
                    "eq": "test",
                    "starts_with": "test",
                },
                "voice_usage_payment_method": "channel",
                "without_tags": "true",
            },
            page_number=0,
            page_size=0,
            sort="connection_name",
        )
        assert_matches_type(AsyncDefaultFlatPagination[PhoneNumberDetailed], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_numbers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[PhoneNumberDetailed], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_numbers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[PhoneNumberDetailed], phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.phone_numbers.delete(
            "1293384261075731499",
        )
        assert_matches_type(PhoneNumberDeleteResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_numbers.with_raw_response.delete(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(PhoneNumberDeleteResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_numbers.with_streaming_response.delete(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(PhoneNumberDeleteResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.phone_numbers.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_slim_list(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.phone_numbers.slim_list()
        assert_matches_type(AsyncDefaultFlatPagination[PhoneNumberSlimListResponse], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_slim_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.phone_numbers.slim_list(
            filter={
                "billing_group_id": "62e4bf2e-c278-4282-b524-488d9c9c43b2",
                "connection_id": "1521916448077776306",
                "country_iso_alpha2": "US",
                "customer_reference": "customer_reference",
                "emergency_address_id": "9102160989215728032",
                "number_type": {"eq": "local"},
                "phone_number": "phone_number",
                "source": "ported",
                "status": "active",
                "tag": "tag",
                "voice_connection_name": {
                    "contains": "test",
                    "ends_with": "test",
                    "eq": "test",
                    "starts_with": "test",
                },
                "voice_usage_payment_method": "channel",
            },
            include_connection=True,
            include_tags=True,
            page_number=0,
            page_size=0,
            sort="connection_name",
        )
        assert_matches_type(AsyncDefaultFlatPagination[PhoneNumberSlimListResponse], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_slim_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_numbers.with_raw_response.slim_list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[PhoneNumberSlimListResponse], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_slim_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_numbers.with_streaming_response.slim_list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[PhoneNumberSlimListResponse], phone_number, path=["response"]
            )

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    SimCardOrder,
    SimCardOrderCreateResponse,
    SimCardOrderRetrieveResponse,
)
from telnyx._utils import parse_datetime
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSimCardOrders:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        sim_card_order = client.sim_card_orders.create(
            address_id="1293384261075731499",
            quantity=23,
        )
        assert_matches_type(SimCardOrderCreateResponse, sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.sim_card_orders.with_raw_response.create(
            address_id="1293384261075731499",
            quantity=23,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_order = response.parse()
        assert_matches_type(SimCardOrderCreateResponse, sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.sim_card_orders.with_streaming_response.create(
            address_id="1293384261075731499",
            quantity=23,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_order = response.parse()
            assert_matches_type(SimCardOrderCreateResponse, sim_card_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        sim_card_order = client.sim_card_orders.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardOrderRetrieveResponse, sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.sim_card_orders.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_order = response.parse()
        assert_matches_type(SimCardOrderRetrieveResponse, sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.sim_card_orders.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_order = response.parse()
            assert_matches_type(SimCardOrderRetrieveResponse, sim_card_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_card_orders.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        sim_card_order = client.sim_card_orders.list()
        assert_matches_type(SyncDefaultFlatPagination[SimCardOrder], sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        sim_card_order = client.sim_card_orders.list(
            filter={
                "address_administrative_area": "TX",
                "address_country_code": "US",
                "address_extended_address": "14th Floor",
                "address_id": "1293384261075731499",
                "address_locality": "Austin",
                "address_postal_code": "78701",
                "address_street_address": "600 Congress Avenue",
                "cost_amount": "2.53",
                "cost_currency": "USD",
                "created_at": parse_datetime("2018-02-02T22:25:27.521Z"),
                "quantity": 21,
                "updated_at": parse_datetime("2018-02-02T22:25:27.521Z"),
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[SimCardOrder], sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.sim_card_orders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_order = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[SimCardOrder], sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.sim_card_orders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_order = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[SimCardOrder], sim_card_order, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSimCardOrders:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        sim_card_order = await async_client.sim_card_orders.create(
            address_id="1293384261075731499",
            quantity=23,
        )
        assert_matches_type(SimCardOrderCreateResponse, sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_orders.with_raw_response.create(
            address_id="1293384261075731499",
            quantity=23,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_order = await response.parse()
        assert_matches_type(SimCardOrderCreateResponse, sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_orders.with_streaming_response.create(
            address_id="1293384261075731499",
            quantity=23,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_order = await response.parse()
            assert_matches_type(SimCardOrderCreateResponse, sim_card_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        sim_card_order = await async_client.sim_card_orders.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardOrderRetrieveResponse, sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_orders.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_order = await response.parse()
        assert_matches_type(SimCardOrderRetrieveResponse, sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_orders.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_order = await response.parse()
            assert_matches_type(SimCardOrderRetrieveResponse, sim_card_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_card_orders.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        sim_card_order = await async_client.sim_card_orders.list()
        assert_matches_type(AsyncDefaultFlatPagination[SimCardOrder], sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card_order = await async_client.sim_card_orders.list(
            filter={
                "address_administrative_area": "TX",
                "address_country_code": "US",
                "address_extended_address": "14th Floor",
                "address_id": "1293384261075731499",
                "address_locality": "Austin",
                "address_postal_code": "78701",
                "address_street_address": "600 Congress Avenue",
                "cost_amount": "2.53",
                "cost_currency": "USD",
                "created_at": parse_datetime("2018-02-02T22:25:27.521Z"),
                "quantity": 21,
                "updated_at": parse_datetime("2018-02-02T22:25:27.521Z"),
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[SimCardOrder], sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_orders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_order = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[SimCardOrder], sim_card_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_orders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_order = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[SimCardOrder], sim_card_order, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["MediaUpdateParams"]


class MediaUpdateParams(TypedDict, total=False):
    media_url: str
    """The URL where the media to be stored in Telnyx network is currently hosted.

    The maximum allowed size is 20 MB.
    """

    ttl_secs: int
    """
    The number of seconds after which the media resource will be deleted, defaults
    to 2 days. The maximum allowed vale is 630720000, which translates to 20 years.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo

__all__ = ["MessagingCreateParams"]


class MessagingCreateParams(TypedDict, total=False):
    aggregation_type: Required[int]
    """Aggregation type: No aggregation = 0, By Messaging Profile = 1, By Tags = 2"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    managed_accounts: SequenceNotStr[str]
    """List of managed accounts to include"""

    profiles: SequenceNotStr[str]
    """List of messaging profile IDs to filter by"""

    select_all_managed_accounts: bool

    start_time: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .room_composition import RoomComposition

__all__ = ["RoomCompositionRetrieveResponse"]


class RoomCompositionRetrieveResponse(BaseModel):
    data: Optional[RoomComposition] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ProfileUpdateParams"]


class ProfileUpdateParams(TypedDict, total=False):
    about: str

    address: str

    category: str

    description: str

    display_name: str

    email: str

    website: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .legacy import (
    LegacyResource,
    AsyncLegacyResource,
    LegacyResourceWithRawResponse,
    AsyncLegacyResourceWithRawResponse,
    LegacyResourceWithStreamingResponse,
    AsyncLegacyResourceWithStreamingResponse,
)
from .reporting import (
    ReportingResource,
    AsyncReportingResource,
    ReportingResourceWithRawResponse,
    AsyncReportingResourceWithRawResponse,
    ReportingResourceWithStreamingResponse,
    AsyncReportingResourceWithStreamingResponse,
)

__all__ = [
    "ReportingResource",
    "AsyncReportingResource",
    "ReportingResourceWithRawResponse",
    "AsyncReportingResourceWithRawResponse",
    "ReportingResourceWithStreamingResponse",
    "AsyncReportingResourceWithStreamingResponse",
    "LegacyResource",
    "AsyncLegacyResource",
    "LegacyResourceWithRawResponse",
    "AsyncLegacyResourceWithRawResponse",
    "LegacyResourceWithStreamingResponse",
    "AsyncLegacyResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["RequirementTypeListParams", "Filter", "FilterName"]


class RequirementTypeListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter for requirement types (deepObject style).

    Originally: filter[name]
    """

    sort: List[Literal["name", "created_at", "updated_at", "-name", "-created_at", "-updated_at"]]
    """Consolidated sort parameter for requirement types (deepObject style).

    Originally: sort[]
    """


class FilterName(TypedDict, total=False):
    contains: str
    """Filters requirement types to those whose name contains a certain string."""


class Filter(TypedDict, total=False):
    """Consolidated filter parameter for requirement types (deepObject style).

    Originally: filter[name]
    """

    name: FilterName



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["VirtualCrossConnectUpdateParams"]


class VirtualCrossConnectUpdateParams(TypedDict, total=False):
    primary_cloud_ip: str
    """
    The IP address assigned for your side of the Virtual Cross
    Connect.<br /><br />If none is provided, one will be generated for
    you.<br /><br />This value can not be patched once the VXC has bene provisioned.
    """

    primary_enabled: bool
    """Indicates whether the primary circuit is enabled.

    Setting this to `false` will disable the circuit.
    """

    primary_routing_announcement: bool
    """Whether the primary BGP route is being announced."""

    secondary_cloud_ip: str
    """
    The IP address assigned for your side of the Virtual Cross
    Connect.<br /><br />If none is provided, one will be generated for
    you.<br /><br />This value can not be patched once the VXC has bene provisioned.
    """

    secondary_enabled: bool
    """Indicates whether the secondary circuit is enabled.

    Setting this to `false` will disable the circuit.
    """

    secondary_routing_announcement: bool
    """Whether the secondary BGP route is being announced."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["PortingOrderRequirement"]


class PortingOrderRequirement(BaseModel):
    field_type: Optional[Literal["document"]] = None
    """Type of value expected on field_value field"""

    field_value: Optional[str] = None
    """identifies the document that satisfies this requirement"""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    requirement_type_id: Optional[str] = None
    """Identifies the requirement type that meets this requirement"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr
from .dtmf_type import DtmfType
from .encrypted_media import EncryptedMedia
from .outbound_ip_param import OutboundIPParam
from .anchorsite_override import AnchorsiteOverride
from .connection_rtcp_settings_param import ConnectionRtcpSettingsParam
from .shared_params.connection_jitter_buffer import ConnectionJitterBuffer
from .shared_params.connection_noise_suppression_details import ConnectionNoiseSuppressionDetails

__all__ = ["IPConnectionCreateParams", "Inbound"]


class IPConnectionCreateParams(TypedDict, total=False):
    active: bool
    """Defaults to true"""

    anchorsite_override: AnchorsiteOverride
    """
    `Latency` directs Telnyx to route media through the site with the lowest
    round-trip time to the user's connection. Telnyx calculates this time using ICMP
    ping messages. This can be disabled by specifying a site to handle all media.
    """

    android_push_credential_id: Optional[str]
    """The uuid of the push credential for Android"""

    call_cost_in_webhooks: bool
    """Specifies if call cost webhooks should be sent for this connection."""

    connection_name: str

    default_on_hold_comfort_noise_enabled: bool
    """When enabled, Telnyx will generate comfort noise when you place the call on
    hold.

    If disabled, you will need to generate comfort noise or on hold music to avoid
    RTP timeout.
    """

    dtmf_type: DtmfType
    """Sets the type of DTMF digits sent from Telnyx to this Connection.

    Note that DTMF digits sent to Telnyx will be accepted in all formats.
    """

    encode_contact_header_enabled: bool
    """
    Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
    scenarios.
    """

    encrypted_media: Optional[EncryptedMedia]
    """Enable use of SRTP for encryption.

    Cannot be set if the transport_portocol is TLS.
    """

    inbound: Inbound

    ios_push_credential_id: Optional[str]
    """The uuid of the push credential for Ios"""

    jitter_buffer: ConnectionJitterBuffer
    """Configuration options for Jitter Buffer.

    Enables Jitter Buffer for RTP streams of SIP Trunking calls. The feature is off
    unless enabled. You may define min and max values in msec for customized
    buffering behaviors. Larger values add latency but tolerate more jitter, while
    smaller values reduce latency but are more sensitive to jitter and reordering.
    """

    noise_suppression: Literal["inbound", "outbound", "both", "disabled"]
    """Controls when noise suppression is applied to calls.

    When set to 'inbound', noise suppression is applied to incoming audio. When set
    to 'outbound', it's applied to outgoing audio. When set to 'both', it's applied
    in both directions. When set to 'disabled', noise suppression is turned off.
    """

    noise_suppression_details: ConnectionNoiseSuppressionDetails
    """Configuration options for noise suppression.

    These settings are stored regardless of the noise_suppression value, but only
    take effect when noise_suppression is not 'disabled'. If you disable noise
    suppression and later re-enable it, the previously configured settings will be
    used.
    """

    onnet_t38_passthrough_enabled: bool
    """
    Enable on-net T38 if you prefer the sender and receiver negotiating T38 directly
    if both are on the Telnyx network. If this is disabled, Telnyx will be able to
    use T38 on just one leg of the call depending on each leg's settings.
    """

    outbound: OutboundIPParam

    rtcp_settings: ConnectionRtcpSettingsParam

    tags: SequenceNotStr[str]
    """Tags associated with the connection."""

    transport_protocol: Literal["UDP", "TCP", "TLS"]
    """One of UDP, TLS, or TCP.

    Applies only to connections with IP authentication or FQDN authentication.
    """

    webhook_api_version: Literal["1", "2"]
    """Determines which webhook format will be used, Telnyx API v1 or v2."""

    webhook_event_failover_url: Optional[str]
    """
    The failover URL where webhooks related to this connection will be sent if
    sending to the primary URL fails. Must include a scheme, such as 'https'.
    """

    webhook_event_url: str
    """The URL where webhooks related to this connection will be sent.

    Must include a scheme, such as 'https'.
    """

    webhook_timeout_secs: Optional[int]
    """Specifies how many seconds to wait before timing out a webhook."""


class Inbound(TypedDict, total=False):
    ani_number_format: Literal["+E.164", "E.164", "+E.164-national", "E.164-national"]
    """
    This setting allows you to set the format with which the caller's number (ANI)
    is sent for inbound phone calls.
    """

    channel_limit: int
    """
    When set, this will limit the total number of inbound calls to phone numbers
    associated with this connection.
    """

    codecs: SequenceNotStr[str]
    """
    Defines the list of codecs that Telnyx will send for inbound calls to a specific
    number on your portal account, in priority order. This only works when the
    Connection the number is assigned to uses Media Handling mode: default. OPUS and
    H.264 codecs are available only when using TCP or TLS transport for SIP.
    """

    default_routing_method: Literal["sequential", "round-robin"]
    """
    Default routing method to be used when a number is associated with the
    connection. Must be one of the routing method types or left blank, other values
    are not allowed.
    """

    dnis_number_format: Literal["+e164", "e164", "national", "sip_username"]

    generate_ringback_tone: bool
    """Generate ringback tone through 183 session progress message with early media."""

    isup_headers_enabled: bool
    """When set, inbound phone calls will receive ISUP parameters via SIP headers.

    (Only when available and only when using TCP or TLS transport.)
    """

    prack_enabled: bool
    """Enable PRACK messages as defined in RFC3262."""

    shaken_stir_enabled: bool
    """
    When enabled the SIP Connection will receive the Identity header with
    Shaken/Stir data in the SIP INVITE message of inbound calls, even when using UDP
    transport.
    """

    sip_compact_headers_enabled: bool
    """Defaults to true."""

    sip_region: Literal["US", "Europe", "Australia"]
    """Selects which `sip_region` to receive inbound calls from.

    If null, the default region (US) will be used.
    """

    sip_subdomain: str
    """
    Specifies a subdomain that can be used to receive Inbound calls to a Connection,
    in the same way a phone number is used, from a SIP endpoint. Example: the
    subdomain "example.sip.telnyx.com" can be called from any SIP endpoint by using
    the SIP URI "sip:@example.sip.telnyx.com" where the user part can be any
    alphanumeric value. Please note TLS encrypted calls are not allowed for
    subdomain calls.
    """

    sip_subdomain_receive_settings: Literal["only_my_connections", "from_anyone"]
    """
    This option can be enabled to receive calls from: "Anyone" (any SIP endpoint in
    the public Internet) or "Only my connections" (any connection assigned to the
    same Telnyx user).
    """

    timeout_1xx_secs: int
    """Time(sec) before aborting if connection is not made."""

    timeout_2xx_secs: int
    """Time(sec) before aborting if call is unanswered (min: 1, max: 600)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .verified_numbers import (
    VerifiedNumbersResource,
    AsyncVerifiedNumbersResource,
    VerifiedNumbersResourceWithRawResponse,
    AsyncVerifiedNumbersResourceWithRawResponse,
    VerifiedNumbersResourceWithStreamingResponse,
    AsyncVerifiedNumbersResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "VerifiedNumbersResource",
    "AsyncVerifiedNumbersResource",
    "VerifiedNumbersResourceWithRawResponse",
    "AsyncVerifiedNumbersResourceWithRawResponse",
    "VerifiedNumbersResourceWithStreamingResponse",
    "AsyncVerifiedNumbersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    IntegrationSecret,
    IntegrationSecretCreateResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestIntegrationSecrets:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        integration_secret = client.integration_secrets.create(
            identifier="my_secret",
            type="bearer",
        )
        assert_matches_type(IntegrationSecretCreateResponse, integration_secret, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        integration_secret = client.integration_secrets.create(
            identifier="my_secret",
            type="bearer",
            token="my_secret_value",
            password="password",
            username="username",
        )
        assert_matches_type(IntegrationSecretCreateResponse, integration_secret, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.integration_secrets.with_raw_response.create(
            identifier="my_secret",
            type="bearer",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        integration_secret = response.parse()
        assert_matches_type(IntegrationSecretCreateResponse, integration_secret, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.integration_secrets.with_streaming_response.create(
            identifier="my_secret",
            type="bearer",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            integration_secret = response.parse()
            assert_matches_type(IntegrationSecretCreateResponse, integration_secret, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        integration_secret = client.integration_secrets.list()
        assert_matches_type(SyncDefaultFlatPagination[IntegrationSecret], integration_secret, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        integration_secret = client.integration_secrets.list(
            filter={"type": "bearer"},
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[IntegrationSecret], integration_secret, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.integration_secrets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        integration_secret = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[IntegrationSecret], integration_secret, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.integration_secrets.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            integration_secret = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[IntegrationSecret], integration_secret, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        integration_secret = client.integration_secrets.delete(
            "id",
        )
        assert integration_secret is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.integration_secrets.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        integration_secret = response.parse()
        assert integration_secret is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.integration_secrets.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            integration_secret = response.parse()
            assert integration_secret is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.integration_secrets.with_raw_response.delete(
                "",
            )


class TestAsyncIntegrationSecrets:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        integration_secret = await async_client.integration_secrets.create(
            identifier="my_secret",
            type="bearer",
        )
        assert_matches_type(IntegrationSecretCreateResponse, integration_secret, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        integration_secret = await async_client.integration_secrets.create(
            identifier="my_secret",
            type="bearer",
            token="my_secret_value",
            password="password",
            username="username",
        )
        assert_matches_type(IntegrationSecretCreateResponse, integration_secret, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.integration_secrets.with_raw_response.create(
            identifier="my_secret",
            type="bearer",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        integration_secret = await response.parse()
        assert_matches_type(IntegrationSecretCreateResponse, integration_secret, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.integration_secrets.with_streaming_response.create(
            identifier="my_secret",
            type="bearer",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            integration_secret = await response.parse()
            assert_matches_type(IntegrationSecretCreateResponse, integration_secret, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        integration_secret = await async_client.integration_secrets.list()
        assert_matches_type(AsyncDefaultFlatPagination[IntegrationSecret], integration_secret, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        integration_secret = await async_client.integration_secrets.list(
            filter={"type": "bearer"},
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[IntegrationSecret], integration_secret, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.integration_secrets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        integration_secret = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[IntegrationSecret], integration_secret, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.integration_secrets.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            integration_secret = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[IntegrationSecret], integration_secret, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        integration_secret = await async_client.integration_secrets.delete(
            "id",
        )
        assert integration_secret is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.integration_secrets.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        integration_secret = await response.parse()
        assert integration_secret is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.integration_secrets.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            integration_secret = await response.parse()
            assert integration_secret is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.integration_secrets.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ....._models import BaseModel
from .telnyx_agent_data import TelnyxAgentData

__all__ = ["TelnyxAgentListResponse"]


class TelnyxAgentListResponse(BaseModel):
    data: List[TelnyxAgentData]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .whatsapp_message_content_param import WhatsappMessageContentParam

__all__ = ["MessageSendWhatsappParams"]


class MessageSendWhatsappParams(TypedDict, total=False):
    from_: Required[Annotated[str, PropertyInfo(alias="from")]]
    """Phone number in +E.164 format associated with Whatsapp account"""

    to: Required[str]
    """Phone number in +E.164 format"""

    whatsapp_message: Required[WhatsappMessageContentParam]

    type: Literal["WHATSAPP"]
    """Message type - must be set to "WHATSAPP" """

    webhook_url: str
    """The URL where webhooks related to this message will be sent."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import Body, Query, Headers, NotGiven, not_given
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.list_retrieve_all_response import ListRetrieveAllResponse
from ..types.list_retrieve_by_zone_response import ListRetrieveByZoneResponse

__all__ = ["ListResource", "AsyncListResource"]


class ListResource(SyncAPIResource):
    """Voice Channels"""

    @cached_property
    def with_raw_response(self) -> ListResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ListResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ListResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ListResourceWithStreamingResponse(self)

    def retrieve_all(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ListRetrieveAllResponse:
        """Retrieve a list of all phone numbers using Channel Billing, grouped by Zone."""
        return self._get(
            "/list",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ListRetrieveAllResponse,
        )

    def retrieve_by_zone(
        self,
        channel_zone_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ListRetrieveByZoneResponse:
        """
        Retrieve a list of phone numbers using Channel Billing for a specific Zone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not channel_zone_id:
            raise ValueError(f"Expected a non-empty value for `channel_zone_id` but received {channel_zone_id!r}")
        return self._get(
            f"/list/{channel_zone_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ListRetrieveByZoneResponse,
        )


class AsyncListResource(AsyncAPIResource):
    """Voice Channels"""

    @cached_property
    def with_raw_response(self) -> AsyncListResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncListResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncListResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncListResourceWithStreamingResponse(self)

    async def retrieve_all(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ListRetrieveAllResponse:
        """Retrieve a list of all phone numbers using Channel Billing, grouped by Zone."""
        return await self._get(
            "/list",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ListRetrieveAllResponse,
        )

    async def retrieve_by_zone(
        self,
        channel_zone_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ListRetrieveByZoneResponse:
        """
        Retrieve a list of phone numbers using Channel Billing for a specific Zone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not channel_zone_id:
            raise ValueError(f"Expected a non-empty value for `channel_zone_id` but received {channel_zone_id!r}")
        return await self._get(
            f"/list/{channel_zone_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ListRetrieveByZoneResponse,
        )


class ListResourceWithRawResponse:
    def __init__(self, list: ListResource) -> None:
        self._list = list

        self.retrieve_all = to_raw_response_wrapper(
            list.retrieve_all,
        )
        self.retrieve_by_zone = to_raw_response_wrapper(
            list.retrieve_by_zone,
        )


class AsyncListResourceWithRawResponse:
    def __init__(self, list: AsyncListResource) -> None:
        self._list = list

        self.retrieve_all = async_to_raw_response_wrapper(
            list.retrieve_all,
        )
        self.retrieve_by_zone = async_to_raw_response_wrapper(
            list.retrieve_by_zone,
        )


class ListResourceWithStreamingResponse:
    def __init__(self, list: ListResource) -> None:
        self._list = list

        self.retrieve_all = to_streamed_response_wrapper(
            list.retrieve_all,
        )
        self.retrieve_by_zone = to_streamed_response_wrapper(
            list.retrieve_by_zone,
        )


class AsyncListResourceWithStreamingResponse:
    def __init__(self, list: AsyncListResource) -> None:
        self._list = list

        self.retrieve_all = async_to_streamed_response_wrapper(
            list.retrieve_all,
        )
        self.retrieve_by_zone = async_to_streamed_response_wrapper(
            list.retrieve_by_zone,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["EmbeddingCreateEmbeddingsParams"]


class EmbeddingCreateEmbeddingsParams(TypedDict, total=False):
    input: Required[Union[str, SequenceNotStr[str]]]
    """Input text to embed. Can be a string or array of strings."""

    model: Required[str]
    """ID of the model to use.

    Use the List embedding models endpoint to see available models.
    """

    dimensions: int
    """The number of dimensions the resulting output embeddings should have.

    Only supported in some models.
    """

    encoding_format: Literal["float", "base64"]
    """The format to return the embeddings in."""

    user: str
    """
    A unique identifier representing your end-user for monitoring and abuse
    detection.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["NumberBlockOrder"]


class NumberBlockOrder(BaseModel):
    id: Optional[str] = None

    connection_id: Optional[str] = None
    """Identifies the connection associated to all numbers in the phone number block."""

    created_at: Optional[datetime] = None
    """An ISO 8901 datetime string denoting when the number order was created."""

    customer_reference: Optional[str] = None
    """A customer reference string for customer look ups."""

    messaging_profile_id: Optional[str] = None
    """
    Identifies the messaging profile associated to all numbers in the phone number
    block.
    """

    phone_numbers_count: Optional[int] = None
    """The count of phone numbers in the number order."""

    range: Optional[int] = None
    """The phone number range included in the block."""

    record_type: Optional[str] = None

    requirements_met: Optional[bool] = None
    """True if all requirements are met for every phone number, false otherwise."""

    starting_number: Optional[str] = None
    """Starting phone number block"""

    status: Optional[Literal["pending", "success", "failure"]] = None
    """The status of the order."""

    updated_at: Optional[datetime] = None
    """An ISO 8901 datetime string for when the number order was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import ota_update_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.ota_update_list_response import OtaUpdateListResponse
from ..types.ota_update_retrieve_response import OtaUpdateRetrieveResponse

__all__ = ["OtaUpdatesResource", "AsyncOtaUpdatesResource"]


class OtaUpdatesResource(SyncAPIResource):
    """OTA updates operations"""

    @cached_property
    def with_raw_response(self) -> OtaUpdatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return OtaUpdatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OtaUpdatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return OtaUpdatesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OtaUpdateRetrieveResponse:
        """
        This API returns the details of an Over the Air (OTA) update.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/ota_updates/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OtaUpdateRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: ota_update_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[OtaUpdateListResponse]:
        """
        List OTA updates

        Args:
          filter:
              Consolidated filter parameter for OTA updates (deepObject style). Originally:
              filter[status], filter[sim_card_id], filter[type]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ota_updates",
            page=SyncDefaultFlatPagination[OtaUpdateListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    ota_update_list_params.OtaUpdateListParams,
                ),
            ),
            model=OtaUpdateListResponse,
        )


class AsyncOtaUpdatesResource(AsyncAPIResource):
    """OTA updates operations"""

    @cached_property
    def with_raw_response(self) -> AsyncOtaUpdatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOtaUpdatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOtaUpdatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncOtaUpdatesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OtaUpdateRetrieveResponse:
        """
        This API returns the details of an Over the Air (OTA) update.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/ota_updates/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OtaUpdateRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: ota_update_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[OtaUpdateListResponse, AsyncDefaultFlatPagination[OtaUpdateListResponse]]:
        """
        List OTA updates

        Args:
          filter:
              Consolidated filter parameter for OTA updates (deepObject style). Originally:
              filter[status], filter[sim_card_id], filter[type]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ota_updates",
            page=AsyncDefaultFlatPagination[OtaUpdateListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    ota_update_list_params.OtaUpdateListParams,
                ),
            ),
            model=OtaUpdateListResponse,
        )


class OtaUpdatesResourceWithRawResponse:
    def __init__(self, ota_updates: OtaUpdatesResource) -> None:
        self._ota_updates = ota_updates

        self.retrieve = to_raw_response_wrapper(
            ota_updates.retrieve,
        )
        self.list = to_raw_response_wrapper(
            ota_updates.list,
        )


class AsyncOtaUpdatesResourceWithRawResponse:
    def __init__(self, ota_updates: AsyncOtaUpdatesResource) -> None:
        self._ota_updates = ota_updates

        self.retrieve = async_to_raw_response_wrapper(
            ota_updates.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            ota_updates.list,
        )


class OtaUpdatesResourceWithStreamingResponse:
    def __init__(self, ota_updates: OtaUpdatesResource) -> None:
        self._ota_updates = ota_updates

        self.retrieve = to_streamed_response_wrapper(
            ota_updates.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            ota_updates.list,
        )


class AsyncOtaUpdatesResourceWithStreamingResponse:
    def __init__(self, ota_updates: AsyncOtaUpdatesResource) -> None:
        self._ota_updates = ota_updates

        self.retrieve = async_to_streamed_response_wrapper(
            ota_updates.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            ota_updates.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["ToolTestParams"]


class ToolTestParams(TypedDict, total=False):
    assistant_id: Required[str]

    arguments: Dict[str, object]
    """Key-value arguments to use for the webhook test"""

    dynamic_variables: Dict[str, object]
    """Key-value dynamic variables to use for the webhook test"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .reports import (
    ReportsResource,
    AsyncReportsResource,
    ReportsResourceWithRawResponse,
    AsyncReportsResourceWithRawResponse,
    ReportsResourceWithStreamingResponse,
    AsyncReportsResourceWithStreamingResponse,
)
from .cdr_usage_reports import (
    CdrUsageReportsResource,
    AsyncCdrUsageReportsResource,
    CdrUsageReportsResourceWithRawResponse,
    AsyncCdrUsageReportsResourceWithRawResponse,
    CdrUsageReportsResourceWithStreamingResponse,
    AsyncCdrUsageReportsResourceWithStreamingResponse,
)
from .mdr_usage_reports import (
    MdrUsageReportsResource,
    AsyncMdrUsageReportsResource,
    MdrUsageReportsResourceWithRawResponse,
    AsyncMdrUsageReportsResourceWithRawResponse,
    MdrUsageReportsResourceWithStreamingResponse,
    AsyncMdrUsageReportsResourceWithStreamingResponse,
)

__all__ = [
    "CdrUsageReportsResource",
    "AsyncCdrUsageReportsResource",
    "CdrUsageReportsResourceWithRawResponse",
    "AsyncCdrUsageReportsResourceWithRawResponse",
    "CdrUsageReportsResourceWithStreamingResponse",
    "AsyncCdrUsageReportsResourceWithStreamingResponse",
    "MdrUsageReportsResource",
    "AsyncMdrUsageReportsResource",
    "MdrUsageReportsResourceWithRawResponse",
    "AsyncMdrUsageReportsResourceWithRawResponse",
    "MdrUsageReportsResourceWithStreamingResponse",
    "AsyncMdrUsageReportsResourceWithStreamingResponse",
    "ReportsResource",
    "AsyncReportsResource",
    "ReportsResourceWithRawResponse",
    "AsyncReportsResourceWithRawResponse",
    "ReportsResourceWithStreamingResponse",
    "AsyncReportsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["VoicemailCreateParams"]


class VoicemailCreateParams(TypedDict, total=False):
    enabled: bool
    """Whether voicemail is enabled."""

    pin: str
    """The pin used for voicemail"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_playback_ended import CallPlaybackEnded

__all__ = ["CallPlaybackEndedWebhookEvent"]


class CallPlaybackEndedWebhookEvent(BaseModel):
    data: Optional[CallPlaybackEnded] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ActionAcceptSuggestionsResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """The UUID of the location."""

    accepted: Optional[bool] = None
    """Indicates if the address suggestions are accepted."""

    record_type: Optional[Literal["address_suggestion"]] = None


class ActionAcceptSuggestionsResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["MessageSendWithAlphanumericSenderParams"]


class MessageSendWithAlphanumericSenderParams(TypedDict, total=False):
    from_: Required[Annotated[str, PropertyInfo(alias="from")]]
    """A valid alphanumeric sender ID on the user's account."""

    messaging_profile_id: Required[str]
    """The messaging profile ID to use."""

    text: Required[str]
    """The message body."""

    to: Required[str]
    """Receiving address (+E.164 formatted phone number)."""

    use_profile_webhooks: bool
    """If true, use the messaging profile's webhook settings."""

    webhook_failover_url: Optional[str]
    """Failover callback URL for delivery status updates."""

    webhook_url: Optional[str]
    """Callback URL for delivery status updates."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .record import Record

__all__ = ["WireguardPeerListResponse"]


class WireguardPeerListResponse(Record):
    last_seen: Optional[str] = None
    """ISO 8601 formatted date-time indicating when peer sent traffic last time."""

    private_key: Optional[str] = None
    """
    Your WireGuard `Interface.PrivateKey`.<br /><br />This attribute is only ever
    utlised if, on POST, you do NOT provide your own `public_key`. In which case, a
    new Public and Private key pair will be generated for you. When your
    `private_key` is returned, you must save this immediately as we do not save it
    within Telnyx. If you lose your Private Key, it can not be recovered.
    """

    wireguard_interface_id: Optional[str] = None
    """The id of the wireguard interface associated with the peer."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import wireless_blocklist_value_list_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.wireless_blocklist_value_list_response import WirelessBlocklistValueListResponse

__all__ = ["WirelessBlocklistValuesResource", "AsyncWirelessBlocklistValuesResource"]


class WirelessBlocklistValuesResource(SyncAPIResource):
    """Wireless Blocklists operations"""

    @cached_property
    def with_raw_response(self) -> WirelessBlocklistValuesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return WirelessBlocklistValuesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WirelessBlocklistValuesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return WirelessBlocklistValuesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        type: Literal["country", "mcc", "plmn"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WirelessBlocklistValueListResponse:
        """
        Retrieve all wireless blocklist values for a given blocklist type.

        Args:
          type: The Wireless Blocklist type for which to list possible values (e.g., `country`,
              `mcc`, `plmn`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/wireless_blocklist_values",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"type": type}, wireless_blocklist_value_list_params.WirelessBlocklistValueListParams
                ),
            ),
            cast_to=WirelessBlocklistValueListResponse,
        )


class AsyncWirelessBlocklistValuesResource(AsyncAPIResource):
    """Wireless Blocklists operations"""

    @cached_property
    def with_raw_response(self) -> AsyncWirelessBlocklistValuesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWirelessBlocklistValuesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWirelessBlocklistValuesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncWirelessBlocklistValuesResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        type: Literal["country", "mcc", "plmn"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WirelessBlocklistValueListResponse:
        """
        Retrieve all wireless blocklist values for a given blocklist type.

        Args:
          type: The Wireless Blocklist type for which to list possible values (e.g., `country`,
              `mcc`, `plmn`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/wireless_blocklist_values",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"type": type}, wireless_blocklist_value_list_params.WirelessBlocklistValueListParams
                ),
            ),
            cast_to=WirelessBlocklistValueListResponse,
        )


class WirelessBlocklistValuesResourceWithRawResponse:
    def __init__(self, wireless_blocklist_values: WirelessBlocklistValuesResource) -> None:
        self._wireless_blocklist_values = wireless_blocklist_values

        self.list = to_raw_response_wrapper(
            wireless_blocklist_values.list,
        )


class AsyncWirelessBlocklistValuesResourceWithRawResponse:
    def __init__(self, wireless_blocklist_values: AsyncWirelessBlocklistValuesResource) -> None:
        self._wireless_blocklist_values = wireless_blocklist_values

        self.list = async_to_raw_response_wrapper(
            wireless_blocklist_values.list,
        )


class WirelessBlocklistValuesResourceWithStreamingResponse:
    def __init__(self, wireless_blocklist_values: WirelessBlocklistValuesResource) -> None:
        self._wireless_blocklist_values = wireless_blocklist_values

        self.list = to_streamed_response_wrapper(
            wireless_blocklist_values.list,
        )


class AsyncWirelessBlocklistValuesResourceWithStreamingResponse:
    def __init__(self, wireless_blocklist_values: AsyncWirelessBlocklistValuesResource) -> None:
        self._wireless_blocklist_values = wireless_blocklist_values

        self.list = async_to_streamed_response_wrapper(
            wireless_blocklist_values.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .global_ip_assignment_param import GlobalIPAssignmentParam

__all__ = ["GlobalIPAssignmentUpdateParams", "GlobalIPAssignmentUpdateRequest"]


class GlobalIPAssignmentUpdateParams(TypedDict, total=False):
    global_ip_assignment_update_request: Required[
        Annotated[GlobalIPAssignmentUpdateRequest, PropertyInfo(alias="globalIpAssignmentUpdateRequest")]
    ]


class GlobalIPAssignmentUpdateRequest(GlobalIPAssignmentParam, total=False):
    pass



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncPerPagePagination, AsyncPerPagePagination
from ....._base_client import AsyncPaginator, make_request_options
from .....types.legacy.reporting.usage_reports import voice_list_params, voice_create_params
from .....types.legacy.reporting.usage_reports.voice_create_response import VoiceCreateResponse
from .....types.legacy.reporting.usage_reports.voice_delete_response import VoiceDeleteResponse
from .....types.legacy.reporting.usage_reports.voice_retrieve_response import VoiceRetrieveResponse
from .....types.legacy.reporting.usage_reports.cdr_usage_report_response_legacy import CdrUsageReportResponseLegacy

__all__ = ["VoiceResource", "AsyncVoiceResource"]


class VoiceResource(SyncAPIResource):
    """Voice usage reports"""

    @cached_property
    def with_raw_response(self) -> VoiceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VoiceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VoiceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VoiceResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        end_time: Union[str, datetime],
        start_time: Union[str, datetime],
        aggregation_type: int | Omit = omit,
        connections: Iterable[int] | Omit = omit,
        managed_accounts: SequenceNotStr[str] | Omit = omit,
        product_breakdown: int | Omit = omit,
        select_all_managed_accounts: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceCreateResponse:
        """
        Creates a new legacy usage V2 CDR report request with the specified filters

        Args:
          end_time: End time in ISO format

          start_time: Start time in ISO format

          aggregation_type: Aggregation type: All = 0, By Connections = 1, By Tags = 2, By Billing Group = 3

          connections: List of connections to filter by

          managed_accounts: List of managed accounts to include

          product_breakdown: Product breakdown type: No breakdown = 0, DID vs Toll-free = 1, Country = 2, DID
              vs Toll-free per Country = 3

          select_all_managed_accounts: Whether to select all managed accounts

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/legacy/reporting/usage_reports/voice",
            body=maybe_transform(
                {
                    "end_time": end_time,
                    "start_time": start_time,
                    "aggregation_type": aggregation_type,
                    "connections": connections,
                    "managed_accounts": managed_accounts,
                    "product_breakdown": product_breakdown,
                    "select_all_managed_accounts": select_all_managed_accounts,
                },
                voice_create_params.VoiceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceRetrieveResponse:
        """
        Fetch single cdr usage report by id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/legacy/reporting/usage_reports/voice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceRetrieveResponse,
        )

    def list(
        self,
        *,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPerPagePagination[CdrUsageReportResponseLegacy]:
        """
        Fetch all previous requests for cdr usage reports.

        Args:
          page: Page number

          per_page: Size of the page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/legacy/reporting/usage_reports/voice",
            page=SyncPerPagePagination[CdrUsageReportResponseLegacy],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    voice_list_params.VoiceListParams,
                ),
            ),
            model=CdrUsageReportResponseLegacy,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceDeleteResponse:
        """
        Deletes a specific V2 legacy usage CDR report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/legacy/reporting/usage_reports/voice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceDeleteResponse,
        )


class AsyncVoiceResource(AsyncAPIResource):
    """Voice usage reports"""

    @cached_property
    def with_raw_response(self) -> AsyncVoiceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVoiceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVoiceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVoiceResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        end_time: Union[str, datetime],
        start_time: Union[str, datetime],
        aggregation_type: int | Omit = omit,
        connections: Iterable[int] | Omit = omit,
        managed_accounts: SequenceNotStr[str] | Omit = omit,
        product_breakdown: int | Omit = omit,
        select_all_managed_accounts: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceCreateResponse:
        """
        Creates a new legacy usage V2 CDR report request with the specified filters

        Args:
          end_time: End time in ISO format

          start_time: Start time in ISO format

          aggregation_type: Aggregation type: All = 0, By Connections = 1, By Tags = 2, By Billing Group = 3

          connections: List of connections to filter by

          managed_accounts: List of managed accounts to include

          product_breakdown: Product breakdown type: No breakdown = 0, DID vs Toll-free = 1, Country = 2, DID
              vs Toll-free per Country = 3

          select_all_managed_accounts: Whether to select all managed accounts

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/legacy/reporting/usage_reports/voice",
            body=await async_maybe_transform(
                {
                    "end_time": end_time,
                    "start_time": start_time,
                    "aggregation_type": aggregation_type,
                    "connections": connections,
                    "managed_accounts": managed_accounts,
                    "product_breakdown": product_breakdown,
                    "select_all_managed_accounts": select_all_managed_accounts,
                },
                voice_create_params.VoiceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceRetrieveResponse:
        """
        Fetch single cdr usage report by id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/legacy/reporting/usage_reports/voice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceRetrieveResponse,
        )

    def list(
        self,
        *,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CdrUsageReportResponseLegacy, AsyncPerPagePagination[CdrUsageReportResponseLegacy]]:
        """
        Fetch all previous requests for cdr usage reports.

        Args:
          page: Page number

          per_page: Size of the page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/legacy/reporting/usage_reports/voice",
            page=AsyncPerPagePagination[CdrUsageReportResponseLegacy],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    voice_list_params.VoiceListParams,
                ),
            ),
            model=CdrUsageReportResponseLegacy,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceDeleteResponse:
        """
        Deletes a specific V2 legacy usage CDR report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/legacy/reporting/usage_reports/voice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceDeleteResponse,
        )


class VoiceResourceWithRawResponse:
    def __init__(self, voice: VoiceResource) -> None:
        self._voice = voice

        self.create = to_raw_response_wrapper(
            voice.create,
        )
        self.retrieve = to_raw_response_wrapper(
            voice.retrieve,
        )
        self.list = to_raw_response_wrapper(
            voice.list,
        )
        self.delete = to_raw_response_wrapper(
            voice.delete,
        )


class AsyncVoiceResourceWithRawResponse:
    def __init__(self, voice: AsyncVoiceResource) -> None:
        self._voice = voice

        self.create = async_to_raw_response_wrapper(
            voice.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            voice.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            voice.list,
        )
        self.delete = async_to_raw_response_wrapper(
            voice.delete,
        )


class VoiceResourceWithStreamingResponse:
    def __init__(self, voice: VoiceResource) -> None:
        self._voice = voice

        self.create = to_streamed_response_wrapper(
            voice.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            voice.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            voice.list,
        )
        self.delete = to_streamed_response_wrapper(
            voice.delete,
        )


class AsyncVoiceResourceWithStreamingResponse:
    def __init__(self, voice: AsyncVoiceResource) -> None:
        self._voice = voice

        self.create = async_to_streamed_response_wrapper(
            voice.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            voice.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            voice.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            voice.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["CallRetrieveCallsParams"]


class CallRetrieveCallsParams(TypedDict, total=False):
    end_time: Annotated[str, PropertyInfo(alias="EndTime")]
    """Filters calls by their end date. Expected format is YYYY-MM-DD"""

    end_time_gt: Annotated[str, PropertyInfo(alias="EndTime_gt")]
    """Filters calls by their end date (after). Expected format is YYYY-MM-DD"""

    end_time_lt: Annotated[str, PropertyInfo(alias="EndTime_lt")]
    """Filters calls by their end date (before). Expected format is YYYY-MM-DD"""

    from_: Annotated[str, PropertyInfo(alias="From")]
    """Filters calls by the from number."""

    page: Annotated[int, PropertyInfo(alias="Page")]
    """
    The number of the page to be displayed, zero-indexed, should be used in
    conjuction with PageToken.
    """

    page_size: Annotated[int, PropertyInfo(alias="PageSize")]
    """The number of records to be displayed on a page"""

    page_token: Annotated[str, PropertyInfo(alias="PageToken")]
    """Used to request the next page of results."""

    start_time: Annotated[str, PropertyInfo(alias="StartTime")]
    """Filters calls by their start date. Expected format is YYYY-MM-DD."""

    start_time_gt: Annotated[str, PropertyInfo(alias="StartTime_gt")]
    """Filters calls by their start date (after). Expected format is YYYY-MM-DD"""

    start_time_lt: Annotated[str, PropertyInfo(alias="StartTime_lt")]
    """Filters calls by their start date (before). Expected format is YYYY-MM-DD"""

    status: Annotated[Literal["canceled", "completed", "failed", "busy", "no-answer"], PropertyInfo(alias="Status")]
    """Filters calls by status."""

    to: Annotated[str, PropertyInfo(alias="To")]
    """Filters calls by the to number."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .payment import (
    PaymentResource,
    AsyncPaymentResource,
    PaymentResourceWithRawResponse,
    AsyncPaymentResourceWithRawResponse,
    PaymentResourceWithStreamingResponse,
    AsyncPaymentResourceWithStreamingResponse,
)
from .auto_recharge_prefs import (
    AutoRechargePrefsResource,
    AsyncAutoRechargePrefsResource,
    AutoRechargePrefsResourceWithRawResponse,
    AsyncAutoRechargePrefsResourceWithRawResponse,
    AutoRechargePrefsResourceWithStreamingResponse,
    AsyncAutoRechargePrefsResourceWithStreamingResponse,
)

__all__ = [
    "AutoRechargePrefsResource",
    "AsyncAutoRechargePrefsResource",
    "AutoRechargePrefsResourceWithRawResponse",
    "AsyncAutoRechargePrefsResourceWithRawResponse",
    "AutoRechargePrefsResourceWithStreamingResponse",
    "AsyncAutoRechargePrefsResourceWithStreamingResponse",
    "PaymentResource",
    "AsyncPaymentResource",
    "PaymentResourceWithRawResponse",
    "AsyncPaymentResourceWithRawResponse",
    "PaymentResourceWithStreamingResponse",
    "AsyncPaymentResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["OtaUpdateListResponse"]


class OtaUpdateListResponse(BaseModel):
    """This object represents an Over the Air (OTA) update request.

    It allows tracking the current status of a operation that apply settings in a particular SIM card. <br/><br/>
    """

    id: Optional[str] = None
    """Identifies the resource."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    record_type: Optional[str] = None

    sim_card_id: Optional[str] = None
    """The identification UUID of the related SIM card resource."""

    status: Optional[Literal["in-progress", "completed", "failed"]] = None

    type: Optional[Literal["sim_card_network_preferences"]] = None
    """Represents the type of the operation requested.

    This will relate directly to the source of the request.
    """

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["PhoneNumbersJobPhoneNumber"]


class PhoneNumbersJobPhoneNumber(BaseModel):
    id: Optional[str] = None
    """The phone number's ID"""

    phone_number: Optional[str] = None
    """The phone number in e164 format."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .auto_recharge_pref import AutoRechargePref

__all__ = ["AutoRechargePrefUpdateResponse"]


class AutoRechargePrefUpdateResponse(BaseModel):
    data: Optional[AutoRechargePref] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from ..test_run_response import TestRunResponse

__all__ = ["RunTriggerResponse"]

RunTriggerResponse: TypeAlias = List[TestRunResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from .brand import (
    BrandResource,
    AsyncBrandResource,
    BrandResourceWithRawResponse,
    AsyncBrandResourceWithRawResponse,
    BrandResourceWithStreamingResponse,
    AsyncBrandResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.messaging_10dlc import campaign_builder_submit_params
from ....types.messaging_10dlc.telnyx_campaign_csp import TelnyxCampaignCsp

__all__ = ["CampaignBuilderResource", "AsyncCampaignBuilderResource"]


class CampaignBuilderResource(SyncAPIResource):
    """Campaign operations"""

    @cached_property
    def brand(self) -> BrandResource:
        """Campaign operations"""
        return BrandResource(self._client)

    @cached_property
    def with_raw_response(self) -> CampaignBuilderResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CampaignBuilderResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CampaignBuilderResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CampaignBuilderResourceWithStreamingResponse(self)

    def submit(
        self,
        *,
        brand_id: str,
        description: str,
        usecase: str,
        age_gated: bool | Omit = omit,
        auto_renewal: bool | Omit = omit,
        direct_lending: bool | Omit = omit,
        embedded_link: bool | Omit = omit,
        embedded_link_sample: str | Omit = omit,
        embedded_phone: bool | Omit = omit,
        help_keywords: str | Omit = omit,
        help_message: str | Omit = omit,
        message_flow: str | Omit = omit,
        mno_ids: Iterable[int] | Omit = omit,
        number_pool: bool | Omit = omit,
        optin_keywords: str | Omit = omit,
        optin_message: str | Omit = omit,
        optout_keywords: str | Omit = omit,
        optout_message: str | Omit = omit,
        privacy_policy_link: str | Omit = omit,
        reference_id: str | Omit = omit,
        reseller_id: str | Omit = omit,
        sample1: str | Omit = omit,
        sample2: str | Omit = omit,
        sample3: str | Omit = omit,
        sample4: str | Omit = omit,
        sample5: str | Omit = omit,
        subscriber_help: bool | Omit = omit,
        subscriber_optin: bool | Omit = omit,
        subscriber_optout: bool | Omit = omit,
        sub_usecases: SequenceNotStr[str] | Omit = omit,
        tag: SequenceNotStr[str] | Omit = omit,
        terms_and_conditions: bool | Omit = omit,
        terms_and_conditions_link: str | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxCampaignCsp:
        """
        Before creating a campaign, use the
        [Qualify By Usecase endpoint](https://developers.telnyx.com/api-reference/campaign/qualify-by-usecase)
        to ensure that the brand you want to assign a new campaign to is qualified for
        the desired use case of that campaign. **Please note:** After campaign creation,
        you'll only be able to edit the campaign's sample messages. Creating a campaign
        will entail an upfront, non-refundable three month's cost that will depend on
        the campaign's use case
        ([see 10DLC Costs section for details](https://developers.telnyx.com/api-reference/campaign/get-campaign-cost)).

        Args:
          brand_id: Alphanumeric identifier of the brand associated with this campaign.

          description: Summary description of this campaign.

          usecase: Campaign usecase. Must be of defined valid types. Use `/10dlc/enum/usecase`
              operation to retrieve usecases available for given brand.

          age_gated: Age gated message content in campaign.

          auto_renewal: Campaign subscription auto-renewal option. If set to true, then campaign will
              automatically renewal at end of billing cycle.

          direct_lending: Direct lending or loan arrangement

          embedded_link: Does message generated by the campaign include URL link in SMS?

          embedded_link_sample: Sample of an embedded link that will be sent to subscribers.

          embedded_phone: Does message generated by the campaign include phone number in SMS?

          help_keywords: Subscriber help keywords. Multiple keywords are comma separated without space.

          help_message: Help message of the campaign.

          message_flow: Message flow description.

          mno_ids: Submit campaign to given list of MNOs by MNO's network ID. Default is all MNOs
              if no value provided.

          number_pool: Does campaign utilize pool of phone numbers?

          optin_keywords: Subscriber opt-in keywords. Multiple keywords are comma separated without space.

          optin_message: Subscriber opt-in message.

          optout_keywords: Subscriber opt-out keywords. Multiple keywords are comma separated without
              space.

          optout_message: Subscriber opt-out message.

          privacy_policy_link: Link to the campaign's privacy policy.

          reference_id: Caller supplied campaign reference ID. If supplied, the value must be unique
              across all submitted campaigns. Can be used to prevent duplicate campaign
              registrations.

          reseller_id: Alphanumeric identifier of the reseller that you want to associate with this
              campaign.

          sample1: Message sample. Some campaign tiers require 1 or more message samples.

          sample2: Message sample. Some campaign tiers require 2 or more message samples.

          sample3: Message sample. Some campaign tiers require 3 or more message samples.

          sample4: Message sample. Some campaign tiers require 4 or more message samples.

          sample5: Message sample. Some campaign tiers require 5 or more message samples.

          subscriber_help: Does campaign responds to help keyword(s)?

          subscriber_optin: Does campaign require subscriber to opt-in before SMS is sent to subscriber?

          subscriber_optout: Does campaign support subscriber opt-out keyword(s)?

          sub_usecases: Campaign sub-usecases. Must be of defined valid sub-usecase types. Use
              `/10dlc/enum/usecase` operation to retrieve list of valid sub-usecases

          tag: Tags to be set on the Campaign.

          terms_and_conditions: Is terms and conditions accepted?

          terms_and_conditions_link: Link to the campaign's terms and conditions.

          webhook_failover_url: Failover webhook to which campaign status updates are sent.

          webhook_url: Webhook to which campaign status updates are sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/10dlc/campaignBuilder",
            body=maybe_transform(
                {
                    "brand_id": brand_id,
                    "description": description,
                    "usecase": usecase,
                    "age_gated": age_gated,
                    "auto_renewal": auto_renewal,
                    "direct_lending": direct_lending,
                    "embedded_link": embedded_link,
                    "embedded_link_sample": embedded_link_sample,
                    "embedded_phone": embedded_phone,
                    "help_keywords": help_keywords,
                    "help_message": help_message,
                    "message_flow": message_flow,
                    "mno_ids": mno_ids,
                    "number_pool": number_pool,
                    "optin_keywords": optin_keywords,
                    "optin_message": optin_message,
                    "optout_keywords": optout_keywords,
                    "optout_message": optout_message,
                    "privacy_policy_link": privacy_policy_link,
                    "reference_id": reference_id,
                    "reseller_id": reseller_id,
                    "sample1": sample1,
                    "sample2": sample2,
                    "sample3": sample3,
                    "sample4": sample4,
                    "sample5": sample5,
                    "subscriber_help": subscriber_help,
                    "subscriber_optin": subscriber_optin,
                    "subscriber_optout": subscriber_optout,
                    "sub_usecases": sub_usecases,
                    "tag": tag,
                    "terms_and_conditions": terms_and_conditions,
                    "terms_and_conditions_link": terms_and_conditions_link,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                campaign_builder_submit_params.CampaignBuilderSubmitParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxCampaignCsp,
        )


class AsyncCampaignBuilderResource(AsyncAPIResource):
    """Campaign operations"""

    @cached_property
    def brand(self) -> AsyncBrandResource:
        """Campaign operations"""
        return AsyncBrandResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCampaignBuilderResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCampaignBuilderResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCampaignBuilderResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCampaignBuilderResourceWithStreamingResponse(self)

    async def submit(
        self,
        *,
        brand_id: str,
        description: str,
        usecase: str,
        age_gated: bool | Omit = omit,
        auto_renewal: bool | Omit = omit,
        direct_lending: bool | Omit = omit,
        embedded_link: bool | Omit = omit,
        embedded_link_sample: str | Omit = omit,
        embedded_phone: bool | Omit = omit,
        help_keywords: str | Omit = omit,
        help_message: str | Omit = omit,
        message_flow: str | Omit = omit,
        mno_ids: Iterable[int] | Omit = omit,
        number_pool: bool | Omit = omit,
        optin_keywords: str | Omit = omit,
        optin_message: str | Omit = omit,
        optout_keywords: str | Omit = omit,
        optout_message: str | Omit = omit,
        privacy_policy_link: str | Omit = omit,
        reference_id: str | Omit = omit,
        reseller_id: str | Omit = omit,
        sample1: str | Omit = omit,
        sample2: str | Omit = omit,
        sample3: str | Omit = omit,
        sample4: str | Omit = omit,
        sample5: str | Omit = omit,
        subscriber_help: bool | Omit = omit,
        subscriber_optin: bool | Omit = omit,
        subscriber_optout: bool | Omit = omit,
        sub_usecases: SequenceNotStr[str] | Omit = omit,
        tag: SequenceNotStr[str] | Omit = omit,
        terms_and_conditions: bool | Omit = omit,
        terms_and_conditions_link: str | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxCampaignCsp:
        """
        Before creating a campaign, use the
        [Qualify By Usecase endpoint](https://developers.telnyx.com/api-reference/campaign/qualify-by-usecase)
        to ensure that the brand you want to assign a new campaign to is qualified for
        the desired use case of that campaign. **Please note:** After campaign creation,
        you'll only be able to edit the campaign's sample messages. Creating a campaign
        will entail an upfront, non-refundable three month's cost that will depend on
        the campaign's use case
        ([see 10DLC Costs section for details](https://developers.telnyx.com/api-reference/campaign/get-campaign-cost)).

        Args:
          brand_id: Alphanumeric identifier of the brand associated with this campaign.

          description: Summary description of this campaign.

          usecase: Campaign usecase. Must be of defined valid types. Use `/10dlc/enum/usecase`
              operation to retrieve usecases available for given brand.

          age_gated: Age gated message content in campaign.

          auto_renewal: Campaign subscription auto-renewal option. If set to true, then campaign will
              automatically renewal at end of billing cycle.

          direct_lending: Direct lending or loan arrangement

          embedded_link: Does message generated by the campaign include URL link in SMS?

          embedded_link_sample: Sample of an embedded link that will be sent to subscribers.

          embedded_phone: Does message generated by the campaign include phone number in SMS?

          help_keywords: Subscriber help keywords. Multiple keywords are comma separated without space.

          help_message: Help message of the campaign.

          message_flow: Message flow description.

          mno_ids: Submit campaign to given list of MNOs by MNO's network ID. Default is all MNOs
              if no value provided.

          number_pool: Does campaign utilize pool of phone numbers?

          optin_keywords: Subscriber opt-in keywords. Multiple keywords are comma separated without space.

          optin_message: Subscriber opt-in message.

          optout_keywords: Subscriber opt-out keywords. Multiple keywords are comma separated without
              space.

          optout_message: Subscriber opt-out message.

          privacy_policy_link: Link to the campaign's privacy policy.

          reference_id: Caller supplied campaign reference ID. If supplied, the value must be unique
              across all submitted campaigns. Can be used to prevent duplicate campaign
              registrations.

          reseller_id: Alphanumeric identifier of the reseller that you want to associate with this
              campaign.

          sample1: Message sample. Some campaign tiers require 1 or more message samples.

          sample2: Message sample. Some campaign tiers require 2 or more message samples.

          sample3: Message sample. Some campaign tiers require 3 or more message samples.

          sample4: Message sample. Some campaign tiers require 4 or more message samples.

          sample5: Message sample. Some campaign tiers require 5 or more message samples.

          subscriber_help: Does campaign responds to help keyword(s)?

          subscriber_optin: Does campaign require subscriber to opt-in before SMS is sent to subscriber?

          subscriber_optout: Does campaign support subscriber opt-out keyword(s)?

          sub_usecases: Campaign sub-usecases. Must be of defined valid sub-usecase types. Use
              `/10dlc/enum/usecase` operation to retrieve list of valid sub-usecases

          tag: Tags to be set on the Campaign.

          terms_and_conditions: Is terms and conditions accepted?

          terms_and_conditions_link: Link to the campaign's terms and conditions.

          webhook_failover_url: Failover webhook to which campaign status updates are sent.

          webhook_url: Webhook to which campaign status updates are sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/10dlc/campaignBuilder",
            body=await async_maybe_transform(
                {
                    "brand_id": brand_id,
                    "description": description,
                    "usecase": usecase,
                    "age_gated": age_gated,
                    "auto_renewal": auto_renewal,
                    "direct_lending": direct_lending,
                    "embedded_link": embedded_link,
                    "embedded_link_sample": embedded_link_sample,
                    "embedded_phone": embedded_phone,
                    "help_keywords": help_keywords,
                    "help_message": help_message,
                    "message_flow": message_flow,
                    "mno_ids": mno_ids,
                    "number_pool": number_pool,
                    "optin_keywords": optin_keywords,
                    "optin_message": optin_message,
                    "optout_keywords": optout_keywords,
                    "optout_message": optout_message,
                    "privacy_policy_link": privacy_policy_link,
                    "reference_id": reference_id,
                    "reseller_id": reseller_id,
                    "sample1": sample1,
                    "sample2": sample2,
                    "sample3": sample3,
                    "sample4": sample4,
                    "sample5": sample5,
                    "subscriber_help": subscriber_help,
                    "subscriber_optin": subscriber_optin,
                    "subscriber_optout": subscriber_optout,
                    "sub_usecases": sub_usecases,
                    "tag": tag,
                    "terms_and_conditions": terms_and_conditions,
                    "terms_and_conditions_link": terms_and_conditions_link,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                campaign_builder_submit_params.CampaignBuilderSubmitParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxCampaignCsp,
        )


class CampaignBuilderResourceWithRawResponse:
    def __init__(self, campaign_builder: CampaignBuilderResource) -> None:
        self._campaign_builder = campaign_builder

        self.submit = to_raw_response_wrapper(
            campaign_builder.submit,
        )

    @cached_property
    def brand(self) -> BrandResourceWithRawResponse:
        """Campaign operations"""
        return BrandResourceWithRawResponse(self._campaign_builder.brand)


class AsyncCampaignBuilderResourceWithRawResponse:
    def __init__(self, campaign_builder: AsyncCampaignBuilderResource) -> None:
        self._campaign_builder = campaign_builder

        self.submit = async_to_raw_response_wrapper(
            campaign_builder.submit,
        )

    @cached_property
    def brand(self) -> AsyncBrandResourceWithRawResponse:
        """Campaign operations"""
        return AsyncBrandResourceWithRawResponse(self._campaign_builder.brand)


class CampaignBuilderResourceWithStreamingResponse:
    def __init__(self, campaign_builder: CampaignBuilderResource) -> None:
        self._campaign_builder = campaign_builder

        self.submit = to_streamed_response_wrapper(
            campaign_builder.submit,
        )

    @cached_property
    def brand(self) -> BrandResourceWithStreamingResponse:
        """Campaign operations"""
        return BrandResourceWithStreamingResponse(self._campaign_builder.brand)


class AsyncCampaignBuilderResourceWithStreamingResponse:
    def __init__(self, campaign_builder: AsyncCampaignBuilderResource) -> None:
        self._campaign_builder = campaign_builder

        self.submit = async_to_streamed_response_wrapper(
            campaign_builder.submit,
        )

    @cached_property
    def brand(self) -> AsyncBrandResourceWithStreamingResponse:
        """Campaign operations"""
        return AsyncBrandResourceWithStreamingResponse(self._campaign_builder.brand)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.external_connections import civic_address_list_params
from ...types.external_connections.civic_address_list_response import CivicAddressListResponse
from ...types.external_connections.civic_address_retrieve_response import CivicAddressRetrieveResponse

__all__ = ["CivicAddressesResource", "AsyncCivicAddressesResource"]


class CivicAddressesResource(SyncAPIResource):
    """External Connections operations"""

    @cached_property
    def with_raw_response(self) -> CivicAddressesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CivicAddressesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CivicAddressesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CivicAddressesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        address_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CivicAddressRetrieveResponse:
        """
        Return the details of an existing Civic Address with its Locations inside the
        'data' attribute of the response.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not address_id:
            raise ValueError(f"Expected a non-empty value for `address_id` but received {address_id!r}")
        return self._get(
            f"/external_connections/{id}/civic_addresses/{address_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CivicAddressRetrieveResponse,
        )

    def list(
        self,
        id: str,
        *,
        filter: civic_address_list_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CivicAddressListResponse:
        """
        Returns the civic addresses and locations from Microsoft Teams.

        Args:
          filter: Filter parameter for civic addresses (deepObject style). Supports filtering by
              country.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/external_connections/{id}/civic_addresses",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"filter": filter}, civic_address_list_params.CivicAddressListParams),
            ),
            cast_to=CivicAddressListResponse,
        )


class AsyncCivicAddressesResource(AsyncAPIResource):
    """External Connections operations"""

    @cached_property
    def with_raw_response(self) -> AsyncCivicAddressesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCivicAddressesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCivicAddressesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCivicAddressesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        address_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CivicAddressRetrieveResponse:
        """
        Return the details of an existing Civic Address with its Locations inside the
        'data' attribute of the response.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not address_id:
            raise ValueError(f"Expected a non-empty value for `address_id` but received {address_id!r}")
        return await self._get(
            f"/external_connections/{id}/civic_addresses/{address_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CivicAddressRetrieveResponse,
        )

    async def list(
        self,
        id: str,
        *,
        filter: civic_address_list_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CivicAddressListResponse:
        """
        Returns the civic addresses and locations from Microsoft Teams.

        Args:
          filter: Filter parameter for civic addresses (deepObject style). Supports filtering by
              country.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/external_connections/{id}/civic_addresses",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"filter": filter}, civic_address_list_params.CivicAddressListParams),
            ),
            cast_to=CivicAddressListResponse,
        )


class CivicAddressesResourceWithRawResponse:
    def __init__(self, civic_addresses: CivicAddressesResource) -> None:
        self._civic_addresses = civic_addresses

        self.retrieve = to_raw_response_wrapper(
            civic_addresses.retrieve,
        )
        self.list = to_raw_response_wrapper(
            civic_addresses.list,
        )


class AsyncCivicAddressesResourceWithRawResponse:
    def __init__(self, civic_addresses: AsyncCivicAddressesResource) -> None:
        self._civic_addresses = civic_addresses

        self.retrieve = async_to_raw_response_wrapper(
            civic_addresses.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            civic_addresses.list,
        )


class CivicAddressesResourceWithStreamingResponse:
    def __init__(self, civic_addresses: CivicAddressesResource) -> None:
        self._civic_addresses = civic_addresses

        self.retrieve = to_streamed_response_wrapper(
            civic_addresses.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            civic_addresses.list,
        )


class AsyncCivicAddressesResourceWithStreamingResponse:
    def __init__(self, civic_addresses: AsyncCivicAddressesResource) -> None:
        self._civic_addresses = civic_addresses

        self.retrieve = async_to_streamed_response_wrapper(
            civic_addresses.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            civic_addresses.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .webhook_portout_new_comment import WebhookPortoutNewComment
from .webhook_portout_status_changed import WebhookPortoutStatusChanged
from .webhook_portout_foc_date_changed import WebhookPortoutFocDateChanged

__all__ = ["EventListResponse"]

EventListResponse: TypeAlias = Union[
    WebhookPortoutStatusChanged, WebhookPortoutNewComment, WebhookPortoutFocDateChanged
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ExternalConnection", "Inbound", "Outbound"]


class Inbound(BaseModel):
    channel_limit: Optional[int] = None
    """
    When set, this will limit the number of concurrent inbound calls to phone
    numbers associated with this connection.
    """


class Outbound(BaseModel):
    channel_limit: Optional[int] = None
    """
    When set, this will limit the number of concurrent outbound calls to phone
    numbers associated with this connection.
    """

    outbound_voice_profile_id: Optional[str] = None
    """Identifies the associated outbound voice profile."""


class ExternalConnection(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the resource."""

    active: Optional[bool] = None
    """Specifies whether the connection can be used."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    credential_active: Optional[bool] = None
    """If the credential associated with this service is active."""

    external_sip_connection: Optional[Literal["zoom", "operator_connect"]] = None
    """The service that will be consuming this connection."""

    inbound: Optional[Inbound] = None

    outbound: Optional[Outbound] = None

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    tags: Optional[List[str]] = None
    """Tags associated with the connection."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was updated."""

    webhook_api_version: Optional[Literal["1", "2"]] = None
    """Determines which webhook format will be used, Telnyx API v1 or v2."""

    webhook_event_failover_url: Optional[str] = None
    """
    The failover URL where webhooks related to this connection will be sent if
    sending to the primary URL fails. Must include a scheme, such as 'https'.
    """

    webhook_event_url: Optional[str] = None
    """The URL where webhooks related to this connection will be sent.

    Must include a scheme, such as 'https'.
    """

    webhook_timeout_secs: Optional[int] = None
    """Specifies how many seconds to wait before timing out a webhook."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["MessagingHostedNumberOrderCheckEligibilityParams"]


class MessagingHostedNumberOrderCheckEligibilityParams(TypedDict, total=False):
    phone_numbers: Required[SequenceNotStr[str]]
    """List of phone numbers to check eligibility"""
